/**HEADER********************************************************************
* 
* Copyright (c) 2010 Avnet Japan Co.,Ltd (NIKE Borad Project.)
* All Rights Reserved
*
**************************************************************************
*
* $FileName: sensor_Shell_Commands.c$
* $Version : 0.0.8.0$
* $Date    : Dec-5-2009$
*
* Comments:
*
*   
*
*END************************************************************************/


#include "NIKE_demo.h"
#include <mqx.h>
#include <string.h>
#include <shell.h>
#include <fio.h>

#include "NIKE_demo_Shell_Commands.h"
#include "Sensor_Task.h"
#include "Signal_Task.h"

#include "sensor_adc_list.h"

#define SENSOR_FUNCTION_DEBUG      0

static int_32  Shell_sensor_stop(int_32 argc, char_ptr argv[] );
static int_32  Shell_set_Action_threshold(int_32 argc, char_ptr argv[] );
static int_32  Shell_get_parameter(int_32 argc, char_ptr argv[] );
static int_32  Shell_set_Sound(int_32 argc, char_ptr argv[] );
static int_32  Shell_set_debugMode(int_32 argc, char_ptr argv[] );
static int_32  Shell_set_mode(int_32 argc, char_ptr argv[] );
static int_32  Shell_test_Sensor(int_32 argc, char_ptr argv[] );

/* Sensor xxxxx Command List */


const SHELL_COMMAND_STRUCT Shell_sensor_shell_commands[] = {
#if SENSOR_FUNCTION_DEBUG
    { "setth",    Shell_set_Action_threshold        },
    { "setso",    Shell_set_Sound                   },
    { "setmode",  Shell_set_mode                    },
    { "getparam", Shell_get_parameter               },
#endif
    { "test",     Shell_test_Sensor                 },
#if SENSOR_FUNCTION_DEBUG
    { "debug",    Shell_set_debugMode               },
#endif
    { NULL,   NULL } 
};

#define SENSOR_COMMAND_ILLEGAL_PRINTF()  printf("Invalid command.  Type 'help' for a list of commands.\n")

/*FUNCTION*-------------------------------------------------------------------
*
* Function Name    :   Shell_sensor_command
* Returned Value   :  int_32 error code
* Comments  :  Reads from a file .
*
*END*---------------------------------------------------------------------*/

int_32  Shell_sensor_command(int_32 argc, char_ptr argv[] )
{
    boolean           print_usage, shorthelp = FALSE;
    int_32            return_code = SHELL_EXIT_SUCCESS;
    uint_32           i;

    print_usage = Shell_check_help_request(argc, argv, &shorthelp );

    if(!print_usage){
//     printf("c= %d, arg= %s, %s, %s\n", argc, argv[0], argv[1], argv[2]);
        if(argc>1){
            for (i=0;Shell_sensor_shell_commands[i].COMMAND != NULL;i++)  {
                if (strcmp(argv[1], Shell_sensor_shell_commands[i].COMMAND) == 0)  {
                    return_code = (*Shell_sensor_shell_commands[i].SHELL_FUNC)(argc, argv);
                    break;
                }
            }
            
            if (Shell_sensor_shell_commands[i].COMMAND == NULL)  {
                SENSOR_COMMAND_ILLEGAL_PRINTF();
            } 
        }
    } 
    return return_code;
} 

/*FUNCTION*-------------------------------------------------------------------
*
* Function Name    :   Shell_set_Action_threshold
* Returned Value   :  int_32 error code
* Comments  :  Reads from a file .
*
*END*---------------------------------------------------------------------*/
static int_32  Shell_set_Action_threshold(int_32 argc, char_ptr argv[] )
{
    int_32    return_code = SHELL_EXIT_SUCCESS;
    boolean   commandEnable = FALSE;
    int_32    actionType=0;
    int_32    threshold=0;
    uint_16   setThreshold =0;

    if(argc == 4){
        actionType = _io_atoi(argv[2]);
        threshold  = _io_atoi(argv[3]);

        printf("actionType = %d,  threshold = %d\n", actionType, threshold);

        if(actionType < SENSOR_3AXIS_ACTION_TYPE_MAX && threshold > 0 && threshold < 0x1000){
            setThreshold = (uint_16)threshold;

            return_code = sensorCtrlSetActionThreshold( actionType, setThreshold);

            if(return_code == SENSOR_OK){
              commandEnable = TRUE;
            }
        }
    }

    if(commandEnable != TRUE){
        printf("Usage: sensor setth [actionType (0-%d)] [threshold (1-4095)]\n", (SENSOR_3AXIS_ACTION_TYPE_MAX-1));
    }

    return return_code;
}


/*FUNCTION*-------------------------------------------------------------------
*
* Function Name    :   Shell_get_Action_param
* Returned Value   :  int_32 error code
* Comments  :  Reads from a file .
*
*END*---------------------------------------------------------------------*/
static int_32  Shell_get_parameter(int_32 argc, char_ptr argv[] )
{
    static SENSOR_PARAM sensorParam;
    boolean   commandEnable = FALSE;
    uint_32   i;
    _mqx_int size;

    if(argc >= 2){
        size = sensorCtrlGetParam(&sensorParam);

        if(size > 0){

            printf("mode = %08x\n", sensorParam.mode);

            for(i=0; i<SENSOR_3AXIS_ACTION_TYPE_SIZE; i++){
                printf("[%d] soundType=%02d, threshold = %04d, \n", 
                                                               i+1, 
                                                               sensorParam.act_param[i].soundType, 
                                                               sensorParam.act_param[i].threshold
                      );
            }
            commandEnable = TRUE;
        }
    }

    if(commandEnable != TRUE){
        printf("Usage: sensor getparam\n");
    }


    return SHELL_EXIT_SUCCESS;

}


/*FUNCTION*-------------------------------------------------------------------
*
* Function Name    :   Shell_set_Sound
* Returned Value   :  int_32 error code
* Comments  :  Reads from a file .
*
*END*---------------------------------------------------------------------*/
static int_32  Shell_set_Sound(int_32 argc, char_ptr argv[] )
{
    int_32            return_code = SHELL_EXIT_SUCCESS;
    boolean   commandEnable = FALSE;
    int_32    actionType=0;
    int_32    soundType=0;
    uint_16   setSound =0;

    if(argc == 4){
        actionType = _io_atoi(argv[2]);
        soundType  = _io_atoi(argv[3]);

        printf("type = %d,  soundType = %d\n", actionType, soundType);

//        if(actionType < SENSOR_3AXIS_ACTION_TYPE_MAX && soundType >= 0 && soundType < SIGNAL_EFFECT_TYPE_MAX){
        if(actionType < SENSOR_3AXIS_ACTION_TYPE_MAX ){

            setSound = (uint_16)soundType;

            return_code = sensorCtrlSetSoundOfAction( actionType, setSound);

            if(return_code == SENSOR_OK){
              commandEnable = TRUE;
            }
        }
    }

    if(commandEnable != TRUE){
        printf("Usage: sensor setso [Type (0-%d)] [SoundType (0-%d)]\n", 
                                          SENSOR_3AXIS_ACTION_TYPE_MAX, 
                                          SIGNAL_EFFECT_TYPE_MAX
              );
    }
    return return_code;
}

/*FUNCTION*-------------------------------------------------------------------
*
* Function Name    :   Shell_set_mode
* Returned Value   :  int_32 error code
* Comments  :  Reads from a file .
*
*END*---------------------------------------------------------------------*/
static int_32  Shell_set_mode(int_32 argc, char_ptr argv[] )
{
    int_32    return_code = SHELL_EXIT_SUCCESS;
    boolean   commandEnable = FALSE;

    return return_code;
}


/*FUNCTION*-------------------------------------------------------------------
*
* Function Name    :   Shell_set_debugMode
* Returned Value   :  int_32 error code
* Comments  :  Reads from a file .
*
*END*---------------------------------------------------------------------*/
static int_32  Shell_set_debugMode(int_32 argc, char_ptr argv[] )
{
    int_32    return_code = SHELL_EXIT_SUCCESS;
    boolean   commandEnable = FALSE;

    if(argc == 3){
      if (strcmp(argv[2], "on") == 0){
        sensorCtrlSetDebugMode(TRUE);
            commandEnable = TRUE;
      } else if(strcmp(argv[2], "off") == 0){
        sensorCtrlSetDebugMode(FALSE);
            commandEnable = TRUE;
      } else {
            /* commandEnable = FALSE; */
      }
    }

    if(commandEnable != TRUE){
        printf("Usage: %s %s f\n", argv[0], argv[1]);
  }

  return return_code;
}

/*FUNCTION*-------------------------------------------------------------------
*
* Function Name    :   Shell_test_Sensor
* Returned Value   :  int_32 error code
* Comments  :  Reads from a file .
*
*END*---------------------------------------------------------------------*/
static int_32  Shell_test_Sensor(int_32 argc, char_ptr argv[] )
{
  boolean           print_usage, shorthelp = FALSE;
  int_32            return_code = SHELL_EXIT_SUCCESS;
  static SENSOR_ADC_VALUE_TYPE data[SENSOR_ADC_LIST_CNANNEL_NUM_LIST];
  int               cnt=0;

  print_usage = Shell_check_help_request(argc, argv, &shorthelp );

  if(!print_usage){
    if(argc == 3){

      if (strcmp(argv[2], "all") == 0){
        printf("\n");

        /* 30s data display */
        while(cnt < 30){
          sensorCtrlGetAdc(data, SENSOR_ADC_LIST_CNANNEL_NUM_LIST);
          printf("X= 0x%x, Y= 0x%x, Z= 0x%x, Temperature= 0x%x\n", data[0], data[1], data[2], data[3]);
          _time_delay(500);
          cnt++;
        }
      } else {
        SENSOR_COMMAND_ILLEGAL_PRINTF();
      }
    } else {
        SENSOR_COMMAND_ILLEGAL_PRINTF();
    }
  }

  return return_code;
}


