#ifndef __adc_h__
#define __adc_h__
/**HEADER********************************************************************
* 
* Copyright (c) 2008 Freescale Semiconductor;
* All Rights Reserved
*
*************************************************************************** 
*
* THIS SOFTWARE IS PROVIDED BY FREESCALE "AS IS" AND ANY EXPRESSED OR 
* IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES 
* OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  
* IN NO EVENT SHALL FREESCALE OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
* INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
* (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR 
* SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) 
* HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
* STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING 
* IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
* THE POSSIBILITY OF SUCH DAMAGE.
*
**************************************************************************
*
* $FileName: adc.h$
* $Version : 3.0.9.0$
* $Date    : May-26-2009$
*
* Comments:
*
*   Generic ADC driver header file
*
*END************************************************************************/
#if MQX_USE_LWEVENTS
	#include <lwevent.h>
#endif
/*----------------------------------------------------------------------*/
/*
**                          CONSTANT DEFINITIONS
*/

#ifdef __cplusplus
extern "C" {
#endif

_mqx_uint _io_adc_install(char_ptr);
pointer _bsp_get_adc_base_address(void);

/* Initialization struct when opening driver */
typedef struct adc_init_struct {
   uint_16   resolution;       /* resolution of ADC driver, the bigger or equal possible value is used to configure ADC peripheral */
} ADC_INIT_STRUCT, _PTR_ ADC_INIT_STRUCT_PTR;

typedef uint_16 ADC_TRIGGER_MASK;
#define ADC_TRIGGER_1                  (0x0001)
#define ADC_TRIGGER_2                  (0x0002)
#define ADC_TRIGGER_3                  (0x0004)
#define ADC_TRIGGER_4                  (0x0008)
#define ADC_TRIGGER_5                  (0x0010)
#define ADC_TRIGGER_6                  (0x0020)
#define ADC_TRIGGER_7                  (0x0040)
#define ADC_TRIGGER_8                  (0x0080)
#define ADC_TRIGGER_9                  (0x0100)
#define ADC_TRIGGER_10                 (0x0200)
#define ADC_TRIGGER_11                 (0x0400)
#define ADC_TRIGGER_12                 (0x0800)
#define ADC_TRIGGER_13                 (0x1000)
#define ADC_TRIGGER_14                 (0x2000)
#define ADC_TRIGGER_15                 (0x4000)
#define ADC_TRIGGER_16                 (0x8000)

/* flags used to initialize every channel */
#define ADC_CHANNEL_MEASURE_LOOP       (0x00) /* measurement never ends, lwevents are set periodically during measurement */
#define ADC_CHANNEL_MEASURE_ONCE       (0x04) /* measurement ends with set lwevent */
#define ADC_CHANNEL_START_TRIGGERED    (0x02) /* after RUN on channel is set, delay is applied (trigger delay) */
#define ADC_CHANNEL_START_NOW          (0x00) /* after RUN on channel is set, no delay is applied and measurement is immediate */
#define ADC_CHANNEL_ACCUMULATE         (0x10) /* accumulate the samples into one result variable, not to the field of values */

typedef struct adc_init_channel_struct {
   uint_16   source;   /* use some of ADC_SOURCE_* definitions defined in CPU specific header file */
   uint_16   flags;            /* initialization flags, combination of above possibilities */
   uint_32   number_samples;   /* the number of samples needed to trigger event */
   uint_32   time_offset;      /* the time offset applied once after channel is started, in microseconds */
   uint_32   time_period;      /* the time slot between two samples in microseconds */
   uint_32   range;            /* the range of result sample(s), NOT USED NOW */
                               /* Note #1: using ADC_CHANNEL_ACCUMULATE, you can achieve finer resolution than specified in ADC */
                               /* Note #2: The result of ADC driver always fits to the range specified here */
                               /* Note #3: if range is too low, then you lose LSbits */
                               /* Note #4: if range is too high, then you get LSbits cleared */
                               /* Note #5: range can be only a number 2^N (in C language: 1 << N) */
   _mqx_uint buffer_size;      /* Size of cache buffer */
   ADC_TRIGGER_MASK trigger;   /* Trigger channel mask that fires this channel */
#if MQX_USE_LWEVENTS
   LWEVENT_STRUCT_PTR complete_event;
#endif
} ADC_INIT_CHANNEL_STRUCT, _PTR_ ADC_INIT_CHANNEL_STRUCT_PTR;

typedef uint_32 ADC_VALUE_TYPE;
typedef struct adc_result_struct {
   ADC_VALUE_TYPE   result;  
} ADC_RESULT_STRUCT, _PTR_ ADC_RESULT_STRUCT_PTR;

/* IOCTL commands used in ADC driver */
#define	IOCTL_ADC_RUN_CHANNEL           (0x10000000)
#define	IOCTL_ADC_FIRE_TRIGGER          (0x10000001)
#define	IOCTL_ADC_STOP_CHANNEL          (0x10000002)
#define	IOCTL_ADC_STOP_CHANNELS         (0x10000003)
#define	IOCTL_ADC_PAUSE_CHANNEL         (0x10000004)
#define	IOCTL_ADC_PAUSE_CHANNELS        (0x10000005)
#define	IOCTL_ADC_RESUME_CHANNEL        (0x10000006)
#define	IOCTL_ADC_RESUME_CHANNELS       (0x10000007)

#if MQX_CPU == 5225 || MQX_CPU == 5223 || MQX_CPU == 5222
  #include "adc_mcf522xx.h"
#elif MQX_CPU == 51128
  #include "adc_mcf51xx.h"
#else
  #error ADC device driver not supported for processor.
#endif


#ifdef __cplusplus
}
#endif

#endif

/* EOF */
