/**HEADER********************************************************************
* 
* Copyright (c) 2008 Freescale Semiconductor;
* All Rights Reserved
*
* Copyright (c) 1989-2008 ARC International;
* All Rights Reserved
*
*************************************************************************** 
*
* THIS SOFTWARE IS PROVIDED BY FREESCALE "AS IS" AND ANY EXPRESSED OR 
* IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES 
* OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  
* IN NO EVENT SHALL FREESCALE OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
* INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
* (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR 
* SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) 
* HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
* STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING 
* IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
* THE POSSIBILITY OF SUCH DAMAGE.
*
**************************************************************************
*
* $FileName: devapi.h$
* $Version : 3.0.4.0$
* $Date    : Mar-11-2009$
*
* Comments:
*
*   This file contains the declarations specific to the USB Device API
*
*END************************************************************************/
#ifndef __devapi_h__
#define __devapi_h__ 1

#include "usb_dev_types.h"  //"types.h"

#define  USB_SEND                         (1)
#define  USB_RECV                         (0)

/* Endpoint types */
#define  USB_CONTROL_ENDPOINT             (0)
#define  USB_ISOCHRONOUS_ENDPOINT         (1)
#define  USB_BULK_ENDPOINT                (2)
#define  USB_INTERRUPT_ENDPOINT           (3)

/* Informational Request/Set Types */
#define  USB_STATUS_DEVICE_STATE          (0x01)
#define  USB_STATUS_INTERFACE             (0x02)
#define  USB_STATUS_ADDRESS               (0x03)
#define  USB_STATUS_CURRENT_CONFIG        (0x04)
#define  USB_STATUS_SOF_COUNT             (0x05)
#define  USB_STATUS_DEVICE                (0x06)
#define  USB_STATUS_TEST_MODE             (0x07)
#define  USB_STATUS_ENDPOINT              (0x10)
#define  USB_STATUS_ENDPOINT_NUMBER_MASK  (0x0F)

#define  USB_TEST_MODE_TEST_PACKET        (0x0400)

/* Available service types */
/* Services 0 through 15 are reserved for endpoints */
#define  USB_SERVICE_EP0                  (0x00)
#define  USB_SERVICE_EP1                  (0x01)
#define  USB_SERVICE_EP2                  (0x02)
#define  USB_SERVICE_EP3                  (0x03)
#define  USB_SERVICE_BUS_RESET            (0x10)
#define  USB_SERVICE_SUSPEND              (0x11)
#define  USB_SERVICE_SOF                  (0x12)
#define  USB_SERVICE_RESUME               (0x13)
#define  USB_SERVICE_SLEEP                (0x14)
#define  USB_SERVICE_SPEED_DETECTION      (0x15)

/***************************************
**
** Data structures
**
*/
typedef struct _usb_xd_queue
{
   struct xd_struct _PTR_ xd_head_ptr;
   struct xd_struct _PTR_ xd_tail_ptr;
} USB_XD_QUEUE, _PTR_ USB_XD_QUEUE_PTR;

/* Callback function storage structure */
typedef struct service_struct {
   uint_8   TYPE;
   void     (_CODE_PTR_ SERVICE)(pointer, boolean, boolean, uint_8_ptr, uint_32);
   struct service_struct _PTR_   NEXT;
} SERVICE_STRUCT, _PTR_ SERVICE_STRUCT_PTR;

typedef struct {
   pointer              PRIVATE;
   void (_CODE_PTR_     FREE)(_usb_device_handle, XD_STRUCT_PTR);
   pointer              XD_FOR_THIS_DTD;
} SCRATCH_STRUCT, _PTR_ SCRATCH_STRUCT_PTR;

typedef struct xd_struct {
   uchar        EP_NUM;          /* Endpoint number */
   uchar        BDIRECTION;      /* Direction : Send/Receive */
   uint_8       EP_TYPE;         /* Type of the endpoint: Ctrl, Isoch, Bulk, 
                                 ** Int 
                                 */
   uchar        BSTATUS;         /* Current transfer status */
   uchar_ptr    WSTARTADDRESS;   /* Address of first byte */
   uint_32      WTOTALLENGTH;    /* Number of bytes to send/recv */
   uint_32      WSOFAR;          /* Number of bytes recv'd so far */
   uint_16      WMAXPACKETSIZE;  /* Max Packet size */
   boolean     	DONT_ZERO_TERMINATE;

   boolean 		ZERO_PACKET;
   uint_32      WPOSITION;  	 /* current position of the buffer */
   uint_32	   	FIFO_SIZE;		 /* To keep track of FIFO length for this ep */
   uint_32	   	IN_FIFO_ADDR;	 /* To keep track of FIFO start address for this ep */
   uint_32	   	OUT_FIFO_ADDR;	 /* To keep track of FIFO start address for this ep */   
   uint_8	   	VECTOR;			 /* Vector of this endpoint */
   uchar 	   	SETUP[8];		 /* Just maintain SETUP packet */

   pointer 		MY_XDPTR;		 /* This will save the xd_ptr send by API during
   								 **	send/ recv request 
   								 */
   uchar_ptr	OUT_TEMP_BUFFER; /* This is required during OUT interrupt to store 
   								 **	OUT data 
   								 */
   boolean      IS_SETUP_PACKET; /* To identify SETUP packet. this is required when 
   								 ** there is setup packet and upper driver does not want 
   								 ** to send the data in the	 data phase, we still need to 
   								 **	set the CMD_OVER to indicate that the processing of 
   								 **	class/vendor specific request is completed.
   								 */
   uint_16					PENDING_XD;	/* counter */
   
   SCRATCH_STRUCT_PTR SCRATCH_PTR;
} XD_STRUCT, _PTR_ XD_STRUCT_PTR;

#define USB_XD_QADD(head,tail,XD)      \
   if ((head) == NULL) {         \
      (head) = (XD);            \
   } else {                      \
      (tail)->SCRATCH_PTR->PRIVATE = (XD);   \
   } /* Endif */                 \
   (tail) = (XD);               \
   (XD)->SCRATCH_PTR->PRIVATE = NULL
   
#define USB_XD_QGET(head,tail,XD)      \
   (XD) = (head);               \
   if (head) {                   \
      (head) = (XD_STRUCT_PTR)((head)->SCRATCH_PTR->PRIVATE);  \
      if ((head) == NULL) {      \
         (tail) = NULL;          \
      } /* Endif */              \
   } /* Endif */
   
#define USB_XD_QUEUE_INIT(xd_queue_ptr)   \
   (xd_queue_ptr)->xd_head_ptr = 0x0;       \
   (xd_queue_ptr)->xd_tail_ptr = 0x0

#define USB_XD_QUEUE_ENQUEUE(xd_queue_ptr, xd_ptr)  \
   USB_XD_QADD ((xd_queue_ptr)->xd_head_ptr, (xd_queue_ptr)->xd_tail_ptr, (xd_ptr))
   
#define USB_XD_QUEUE_DEQUEUE(xd_queue_ptr, xd_ptr_ptr)  \
   USB_XD_QGET ((xd_queue_ptr)->xd_head_ptr, (xd_queue_ptr)->xd_tail_ptr, *(xd_ptr_ptr))
   
#define USB_XD_QUEUE_ENQUEUE_SAFE(xd_queue_ptr, xd_ptr)  \
   _int_disable ();  \
   USB_XD_QUEUE_ENQUEUE (xd_queue_ptr, xd_ptr); \
   _int_enable ();

#define USB_XD_QUEUE_DEQUEUE_SAFE(xd_queue_ptr, xd_ptr)  \
   _int_disable ();  \
   USB_XD_QUEUE_DEQUEUE (xd_queue_ptr, xd_ptr); \
   _int_enable ();

#define USB_XD_QUEUE_GET_HEAD(xd_queue_ptr, xd_ptr_ptr)  \
   *(xd_ptr_ptr) = (xd_queue_ptr)->xd_head_ptr

#define USB_XD_QUEUE_GET_TAIL(xd_queue_ptr, xd_ptr_ptr)  \
   *(xd_ptr_ptr) = (xd_queue_ptr)->xd_tail_ptr

typedef struct _usb_ep_info_struct
{
   USB_XD_QUEUE xd_queue;  /* FIFO queue for all XDs on this endpoint */
   uint_8 type;
   uint_8 direction; /* for _usb_device_call_service */
   uint_16 max_packet_size;
   uint_16 fifo_length;
   
   uint_8 tx_data0;
   uint_8 rx_data0;
   
   uint_8 tx_buf_odd;
   uint_8 rx_buf_odd;   // next buffer is odd
} USB_EP_INFO_STRUCT, _PTR_ USB_EP_INFO_STRUCT_PTR;

/* The USB Device State Structure */
typedef struct _usb_device_state_struct {
   pointer                          DEV_PTR;             /* Device Controller 
                                                         ** Register base 
                                                         ** address 
                                                         */
   pointer                          CALLBACK_STRUCT_PTR;
   
   SERVICE_STRUCT_PTR               SERVICE_HEAD_PTR;       /* Head struct 
                                                         ** address of 
                                                         ** registered services 
                                                         */
   XD_STRUCT_PTR                    TEMP_XD_PTR;         /* Temp xd for ep init */
   XD_STRUCT_PTR                    XD_BASE;
   XD_STRUCT_PTR                    XD_HEAD;             /* Head Transaction 
                                                         ** descriptors 
                                                         */
   XD_STRUCT_PTR                    XD_TAIL;             /* Tail Transaction 
                                                         ** descriptors 
                                                         */
   XD_STRUCT_PTR                    PENDING_XD_PTR;      /* pending transfer */
   uint_32                          XD_ENTRIES;

   SCRATCH_STRUCT_PTR               XD_SCRATCH_STRUCT_BASE;
   SCRATCH_STRUCT_PTR               SCRATCH_STRUCT_BASE;
   
   uint_16                          USB_STATE;
   uint_16                          USB_DEVICE_STATE;
   uint_16                          USB_SOF_COUNT;
   uint_16                          ERRORS;
   uint_16                          USB_DEV_STATE_B4_SUSPEND;
   uint_8                           DEV_NUM;             /* USB device number 
                                                         ** on the board 
                                                         */
   uint_8                           DEV_VEC;             /* Interrupt vector 
                                                         ** number for USB 
                                                         */
   uint_8                           SPEED;               /* Low Speed, 
                                                         ** High Speed, 
                                                         ** Full Speed 
                                                         */
   uint_8                           MAX_ENDPOINTS;       /* Max endpoints
                                                         ** supported by this
                                                         ** device
                                                         */
                                                         
   uint_8                           USB_CURR_CONFIG;                                                         
   uint_8                           DEVICE_ADDRESS;

   USB_XD_QUEUE                     XD_FREE_LIST;
//   XD_STRUCT			   			   EP[USB_MAX_ENDPOINTS];
   USB_EP_INFO_STRUCT               EP_INFO[USB_MAX_ENDPOINTS];
//   uint_8                           SETUP_PACKET[8];
//   uint_32                          ep0frcfgr;
} USB_DEV_STATE_STRUCT, _PTR_ USB_DEV_STATE_STRUCT_PTR;

typedef pointer _usb_device_handle;

/*
**
** Low-level function list structure for USB
**
** This is the structure used to store chip specific functions to be called 
** by generic layer
*/
typedef struct usb_chip_function_struct
{
   /* The device number */
   uint_32             DEV_NUM;
   
   /* The Host/Device init function */
   uint_8 (_CODE_PTR_ DEV_INIT)(uint_8, _usb_device_handle);

   /* The function to send data */
   uint_8 (_CODE_PTR_ DEV_SEND)(_usb_device_handle, XD_STRUCT_PTR);

   /* The function to receive data */
   uint_8 (_CODE_PTR_ DEV_RECV)(_usb_device_handle, XD_STRUCT_PTR);
   
   /* The function to cancel the transfer */
   uint_8 (_CODE_PTR_ DEV_CANCEL_TRANSFER)(_usb_device_handle, uint_8, uint_8);
   
   uint_8 (_CODE_PTR_ DEV_INIT_ENDPOINT)(_usb_device_handle, XD_STRUCT_PTR);
   
   uint_8 (_CODE_PTR_ DEV_DEINIT_ENDPOINT)(_usb_device_handle, uint_8, uint_8);
   
   uint_8 (_CODE_PTR_ DEV_UNSTALL_ENDPOINT)(_usb_device_handle, uint_8, uint_8);
   
   uint_8 (_CODE_PTR_ DEV_GET_ENDPOINT_STATUS)(_usb_device_handle, uint_8);
   
   uint_8 (_CODE_PTR_ DEV_SET_ENDPOINT_STATUS)(_usb_device_handle, uint_8, uint_8);

   uint_8 (_CODE_PTR_ DEV_GET_TRANSFER_STATUS)(_usb_device_handle, uint_8, uint_8);
   
   uint_8 (_CODE_PTR_ DEV_SET_ADDRESS)(_usb_device_handle, uint_8);
   
   uint_8 (_CODE_PTR_ DEV_SHUTDOWN)(_usb_device_handle);
   
   uint_8 (_CODE_PTR_ DEV_GET_SETUP_DATA)(_usb_device_handle, uint_8, uchar_ptr);
   
   uint_8 (_CODE_PTR_ DEV_ASSERT_RESUME)(_usb_device_handle);
   
   uint_8 (_CODE_PTR_ DEV_STALL_ENDPOINT)(_usb_device_handle, uint_8, uint_8);

   uint_8 (_CODE_PTR_ DEV_SET_TEST_MODE)(_usb_device_handle, uint_8);
} USB_CALLBACK_FUNCTIONS_STRUCT, _PTR_ USB_CALLBACK_FUNCTIONS_STRUCT_PTR;

#include "dev_cncl.h"
#include "dev_ep_deinit.h"
#include "dev_main.h"
#include "dev_recv.h"
#include "dev_send.h"
#include "dev_shut.h"
#include "dev_utl.h"

extern USB_CALLBACK_FUNCTIONS_STRUCT _bsp_usb_callback_table;

#ifdef __cplusplus
extern "C" {
#endif

extern uint_8 _bsp_get_usb_vector(uint_8);
extern pointer _bsp_get_usb_base(uint_8);
   
#ifdef __cplusplus
}
#endif   

#endif
/* EOF */

