/**HEADER********************************************************************
* 
* Copyright (c) 2008 Freescale Semiconductor;
* All Rights Reserved
*
* Copyright (c) 2004-2008 Embedded Access Inc.;
* All Rights Reserved
*
* Copyright (c) 1989-2008 ARC International;
* All Rights Reserved
*
*************************************************************************** 
*
* THIS SOFTWARE IS PROVIDED BY FREESCALE "AS IS" AND ANY EXPRESSED OR 
* IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES 
* OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  
* IN NO EVENT SHALL FREESCALE OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
* INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
* (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR 
* SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) 
* HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
* STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING 
* IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
* THE POSSIBILITY OF SUCH DAMAGE.
*
**************************************************************************
*
* $FileName: qspi_polled.c$
* $Version : 3.0.2.0$
* $Date    : Jun-29-2009$
*
* Comments:
*
*   This file contains the qspi polled I/O (master) driver functions.
*
*END************************************************************************/

#include "mqx_inc.h"
#include "fio.h"
#include "fio_prv.h"
#include "io.h"
#include "io_prv.h"
#include "serial.h"
#include "qspi.h"
#include "qspi_polled_prv.h"


/*FUNCTION*-------------------------------------------------------------------
* 
* Function Name    : _io_qspi_polled_install
* Returned Value   : MQX_OK or a task error code
* Comments         :
*    Install the qspi device.
*
*END*----------------------------------------------------------------------*/

_mqx_uint _io_qspi_polled_install
   (
      /* [IN] A string that identifies the device for fopen */
      char_ptr            	identifier,
  
      /* [IN] The I/O init function */
      _mqx_uint (_CODE_PTR_ init)(pointer, pointer _PTR_, char _PTR_),
      
      /* [IN] The I/O de-init function */
      _mqx_uint (_CODE_PTR_ deinit)(pointer, pointer),
      
      /* [IN] The input function */
      _mqx_int (_CODE_PTR_ recv)(pointer, char _PTR_, _mqx_int),
     
      /* [IN] The output function */
      _mqx_int (_CODE_PTR_ xmit)(pointer, char _PTR_, _mqx_int),

      /* [IN] The I/O ioctl function */
      _mqx_int (_CODE_PTR_ ioctl)(pointer, _mqx_uint, _mqx_uint_ptr),

      /* [IN] The I/O init data pointer */
      pointer				init_data_ptr
      
   )

{ /* Body */
   IO_QSPI_POLLED_DEVICE_STRUCT_PTR polled_dev_ptr;

   polled_dev_ptr = (IO_QSPI_POLLED_DEVICE_STRUCT_PTR)_mem_alloc_system_zero((_mem_size)
      sizeof(IO_QSPI_POLLED_DEVICE_STRUCT));

#if MQX_CHECK_MEMORY_ALLOCATION_ERRORS
   if (polled_dev_ptr == NULL) {
      return(MQX_OUT_OF_MEMORY);
   } /* Endif */
#endif

   polled_dev_ptr->DEV_INIT          = init;
   polled_dev_ptr->DEV_DEINIT        = deinit;
   polled_dev_ptr->DEV_READ          = recv;
   polled_dev_ptr->DEV_WRITE         = xmit;
   polled_dev_ptr->DEV_IOCTL         = ioctl;
   polled_dev_ptr->DEV_INIT_DATA_PTR = init_data_ptr;
   
   return (_io_dev_install_ext(identifier,
      _io_qspi_polled_open, _io_qspi_polled_close,
      _io_qspi_polled_read, _io_qspi_polled_write,
      _io_qspi_polled_ioctl, _io_qspi_polled_uninstall,
      (pointer)polled_dev_ptr));

} /* Endbody */

/*FUNCTION*-------------------------------------------------------------------
*
* Function Name    : _io_qspi_polled_uninstall
* Returned Value   : MQX_OK or a task error code
* Comments         :
*    Un-Install a polled serial device.
*
*END*----------------------------------------------------------------------*/
_mqx_int _io_qspi_polled_uninstall
   (
      /* [IN] The IO device structure for the device */
      IO_DEVICE_STRUCT_PTR   io_dev_ptr
   )
{ /* Body */
   IO_QSPI_POLLED_DEVICE_STRUCT_PTR polled_dev_ptr = io_dev_ptr->DRIVER_INIT_PTR;

   if (polled_dev_ptr->COUNT == 0) {
      if (polled_dev_ptr->DEV_DEINIT) {
         (*polled_dev_ptr->DEV_DEINIT)(polled_dev_ptr->DEV_INIT_DATA_PTR,
            polled_dev_ptr->DEV_INFO_PTR);
      } /* Endif */
      _mem_free(polled_dev_ptr);
      io_dev_ptr->DRIVER_INIT_PTR = NULL;
      return(IO_OK);
   } else {
      return(IO_ERROR_DEVICE_BUSY);
   } /* Endif */

} /* Endbody */

/*FUNCTION****************************************************************
* 
* Function Name    : _io_qspi_polled_open
* Returned Value   : MQX_OK or a task error code
* Comments         :
*    This routine initializes the QSPI I/O channel. It acquires
*    memory, then stores information into it about the channel.
*    It then calls the hardware interface function to initialize the channel.
* 
*END**********************************************************************/
_mqx_int _io_qspi_polled_open
   (
      /* [IN] the file handle for the device being opened */
      FILE_DEVICE_STRUCT_PTR         fd_ptr,
       
      /* [IN] the remaining portion of the name of the device */
      char                     _PTR_ open_name_ptr,

      /* [IN] the flags to be used during operation:
      ** echo, translation, xon/xoff.
      */
      char                     _PTR_ flags
   )
{ /* Body */
   IO_DEVICE_STRUCT_PTR                io_dev_ptr;
   IO_QSPI_POLLED_DEVICE_STRUCT_PTR    polled_dev_ptr;
   _mqx_int                            result = MQX_OK;

   io_dev_ptr     = (IO_DEVICE_STRUCT_PTR)fd_ptr->DEV_PTR;
   polled_dev_ptr = (IO_QSPI_POLLED_DEVICE_STRUCT_PTR)(io_dev_ptr->DRIVER_INIT_PTR);
   
   if (polled_dev_ptr->COUNT) {
      /* Device is already opened */
      polled_dev_ptr->COUNT++;
      fd_ptr->FLAGS = polled_dev_ptr->FLAGS;
      return(result);
   } /* Endif */
      
   polled_dev_ptr->FLAGS = (_mqx_uint)flags;
   fd_ptr->FLAGS         = (_mqx_uint)flags;
            
   result = (*polled_dev_ptr->DEV_INIT)((pointer)polled_dev_ptr->DEV_INIT_DATA_PTR, 
                                        &polled_dev_ptr->DEV_INFO_PTR, open_name_ptr);
   if (result == MQX_OK) {
      polled_dev_ptr->COUNT = 1;
      polled_dev_ptr->RX_BUFF = NULL;
   } /* Endif */
   
   return(result);

} /* Endbody */

/*FUNCTION****************************************************************
* 
* Function Name    : _io_qspi_polled_close
* Returned Value   : MQX_OK or a task error code
* Comments         :
*    This routine closes the QSPI I/O channel.
* 
*END**********************************************************************/
_mqx_int _io_qspi_polled_close
   (
      /* [IN] the file handle for the device being closed */
      FILE_DEVICE_STRUCT_PTR fd_ptr
   )
{ /* Body */
   IO_DEVICE_STRUCT_PTR             io_dev_ptr;
   IO_QSPI_POLLED_DEVICE_STRUCT_PTR polled_dev_ptr;
   _mqx_int                         result = MQX_OK;

   io_dev_ptr     = (IO_DEVICE_STRUCT_PTR)fd_ptr->DEV_PTR;
   polled_dev_ptr = (pointer)io_dev_ptr->DRIVER_INIT_PTR;

   if (--polled_dev_ptr->COUNT == 0) {
      if (polled_dev_ptr->DEV_DEINIT) {
         result = (*polled_dev_ptr->DEV_DEINIT)(polled_dev_ptr->DEV_INIT_DATA_PTR,
            polled_dev_ptr->DEV_INFO_PTR);
      } /* Endif */ 
   } /* Endif */
   return(result);

} /* Endbody */
  
/*FUNCTION****************************************************************
* 
* Function Name    : _io_qspi_polled_read
* Returned Value   : Number of bytes read
* Comments         :
*    Reads bytes into array from QSPI input buffers.
*    Returns number of bytes received.
*
*END*********************************************************************/
_mqx_int _io_qspi_polled_read
   (
      /* [IN] the handle returned from _fopen */
      FILE_DEVICE_STRUCT_PTR          fd_ptr,

      /* [IN] where the characters are to be stored */
      char_ptr                        data_ptr,

      /* [IN] the number of bytes to read */
      _mqx_int                        n
   )
{  /* Body */

   IO_DEVICE_STRUCT_PTR                  io_dev_ptr;
   IO_QSPI_POLLED_DEVICE_STRUCT_PTR      polled_dev_ptr;
   _mqx_int                              num = 0;
   
   io_dev_ptr     = (IO_DEVICE_STRUCT_PTR)fd_ptr->DEV_PTR;
   polled_dev_ptr = (pointer)io_dev_ptr->DRIVER_INIT_PTR;

   num = (*polled_dev_ptr->DEV_READ)(polled_dev_ptr, data_ptr, n);

   return(num);

}  /* Endbody */

/*FUNCTION****************************************************************
* 
* Function Name    : _io_qspi_polled_write
* Returned Value   : Number of bytes output
* Comments         :
*    This routine calls the appropriate write routine.
*
*END**********************************************************************/
_mqx_int _io_qspi_polled_write
   (
      /* [IN] the handle returned from _fopen */
      FILE_DEVICE_STRUCT_PTR    fd_ptr,

      /* [IN] where the bytes are stored */
      char_ptr                  data_ptr,

      /* [IN] the number of bytes to output */
      _mqx_int					n
   )
{  /* Body */
   IO_DEVICE_STRUCT_PTR                  io_dev_ptr;
   IO_QSPI_POLLED_DEVICE_STRUCT_PTR      polled_dev_ptr;
   _mqx_int                              num = 0;
   
   io_dev_ptr     = (IO_DEVICE_STRUCT_PTR)fd_ptr->DEV_PTR;
   polled_dev_ptr = (pointer)io_dev_ptr->DRIVER_INIT_PTR;
               
   num = (*polled_dev_ptr->DEV_WRITE)(polled_dev_ptr, data_ptr, n);
   
   return(num);
              
} /* Endbody */
   
/*FUNCTION*****************************************************************
* 
* Function Name    : _io_qspi_polled_ioctl
* Returned Value   : _mqx_int
* Comments         :
*    Returns result of qspi ioctl operation.
*
*END*********************************************************************/
_mqx_int _io_qspi_polled_ioctl
   (
      /* [IN] the file handle for the device */
      FILE_DEVICE_STRUCT_PTR fd_ptr,

      /* [IN] the ioctl command */
      _mqx_uint              cmd,

      /* [IN] the ioctl parameters */
      pointer                input_param_ptr
   )
{ /* Body */
	IO_DEVICE_STRUCT_PTR               io_dev_ptr;
	IO_QSPI_POLLED_DEVICE_STRUCT_PTR   polled_dev_ptr;
	_mqx_int                           result = MQX_OK;
	_mqx_uint_ptr                      param_ptr = (_mqx_uint_ptr)input_param_ptr;

	io_dev_ptr     = (IO_DEVICE_STRUCT_PTR)fd_ptr->DEV_PTR;
	polled_dev_ptr = (pointer)io_dev_ptr->DRIVER_INIT_PTR;

	switch (cmd) {
    	case IO_IOCTL_SERIAL_GET_FLAGS:
        	*param_ptr = polled_dev_ptr->FLAGS;
         	break;

      	case IO_IOCTL_SERIAL_SET_FLAGS:
         	polled_dev_ptr->FLAGS = *param_ptr;
         	fd_ptr->FLAGS = *param_ptr;
         	break;
   	  
		case IO_IOCTL_DEVICE_IDENTIFY :
			/* return the device identify */
			param_ptr[0] = IO_DEV_TYPE_PHYS_QSPI_POLLED;
			param_ptr[1] = 0;
			param_ptr[2] = IO_DEV_ATTR_POLL | IO_DEV_ATTR_READ | IO_DEV_ATTR_WRITE;
			result = MQX_OK;	
			break;
    		
		default:
         	if (polled_dev_ptr->DEV_IOCTL != NULL) {
           		result = (*polled_dev_ptr->DEV_IOCTL)(polled_dev_ptr->DEV_INFO_PTR, 
              	cmd, param_ptr);
         	} /* Endif */
         	break;             
	} /* Endswitch */
	return(result);

} /* Endbody */

/* EOF */