;*HEADER********************************************************************
;* 
;* Copyright (c) 2008 Freescale Semiconductor;
;* All Rights Reserved                       
;*
;* Copyright (c) 1989-2008 ARC International;
;* All Rights Reserved
;*
;*************************************************************************** 
;*
;* THIS SOFTWARE IS PROVIDED BY FREESCALE "AS IS" AND ANY EXPRESSED OR 
;* IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES 
;* OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  
;* IN NO EVENT SHALL FREESCALE OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
;* INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
;* (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR 
;* SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) 
;* HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
;* STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING 
;* IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
;* THE POSSIBILITY OF SUCH DAMAGE.
;*
;**************************************************************************
;*
;* $FileName: psp_prv.cw$
;* $Version : 3.0.6.0$
;* $Date    : Mar-11-2009$
;*
;* Comments:
;*
;*   This assembler header file contains private declarations for 
;*   use with the mqx assembler files
;*
;*END***********************************************************************


;------------------------------------------------------------------------
;                       CONSTANT DECLARATIONS
;

; The following are the bits in the CONFIG field of the kernel data structure
; to set for the psp options.
; NOTE: These must agree with values in mqx_prv.h

#define __ASM__
#include "mqx_prv.h"



; MUST match source/include/mqx_prv.h
PSP_CNFG2_PSP_ASM_KERNEL_LOGGING             .equ  MQX_CNFG2_PSP_ASM_KERNEL_LOGGING
PSP_CNFG2_PSP_ASM_FP_REGISTERS_EXIST         .equ  MQX_CNFG2_PSP_ASM_FP_REGISTERS_EXIST
PSP_CNFG2_PSP_ASM_RUN_TIME_ERR_CHECK_ENABLED .equ  MQX_CNFG2_PSP_ASM_RUN_TIME_ERR_CHECK_ENABLED
PSP_CNFG2_PSP_ASM_DSP_REGISTERS_EXIST        .equ  MQX_CNFG2_PSP_ASM_DSP_REGISTERS_EXIST

; TODO remove, maybe...
.ifndef PSP_HAS_EMAC
PSP_HAS_EMAC .equ 0
.endif

.ifndef PSP_HAS_DSP
   .if PSP_HAS_EMAC
PSP_HAS_DSP        			     .equ  1
   .else
      .if (MQX_CPU == 5222 || MQX_CPU == 5249 || MQX_CPU == 5282 || MQX_CPU ==  5251)
PSP_HAS_DSP        			     .equ  1
      .else
PSP_HAS_DSP        			     .equ  0
      .endif
   .endif
.endif


.set PSP_CNFG, 0
   .if MQX_KERNEL_LOGGING
.set PSP_CNFG, PSP_CNFG + PSP_CNFG2_PSP_ASM_KERNEL_LOGGING
   .endif
   .if MQX_FP_REGISTERS_EXIST
.set PSP_CNFG, PSP_CNFG + PSP_CNFG2_PSP_ASM_FP_REGISTERS_EXIST
   .endif


; Offset to the stack frame format and vector word from the stack pointer.
; 15 registers slots are allocated before the SR and PC on the stack

FLINT_SR_OFFSET         .equ        4*15+2
FLINT_FORMAT_OFFSET     .equ        FLINT_SR_OFFSET-2
FLINT_PC_OFFSET         .equ        FLINT_FORMAT_OFFSET+4

;  Task FLAGS bits
;  These must match definitions in mqx_prv.h

; MUST match
#if PSP_HAS_FPU 
FP_TASK_MASK            .equ       MQX_FLOATING_POINT_TASK              ; mqx.h -> mqx_prv.h
#endif

FP_CONTEXT_SAVED_MASK   .equ       TASK_FLOATING_POINT_CONTEXT_SAVED    ; mqx_prv.h
FP_CONTEXT_CLEAR_MASK   .equ       0xfdff
PREEMPTION_DISABLED     .equ       TASK_PREEMPTION_DISABLED             ; mqx_prv.h

; DSP register offsets from Stack Base
   .if PSP_HAS_DSP
TD_DSP_MASK     .equ -4
TD_DSP_MACSR    .equ -8
TD_DSP_ACCEXT23 .equ -12
TD_DSP_ACCEXT01 .equ -16
TD_DSP_ACC3     .equ -20
TD_DSP_ACC2     .equ -24
TD_DSP_ACC1     .equ -28
TD_DSP_ACC0     .equ -32
   .endif

;------------------------------------------------------------------------
;                          MACRO DECLARATIONS
;

; This macro returns the address of the kernel data in the specified register
GET_KERNEL_DATA: .macro REG
        move.l  __mqx_kernel_data,REG    ; get the kernel data pointer
        .endm

; Start CR 1169
; This macro returns the address of the system task stack ptr in the specified register
GET_SYSTEM_STACK: .macro REG
		move.l  __mqx_system_stack, REG
		.endm
; End CR 1169

; This macro saves the context for the running task when called from a C 
; function. Since it is called from a C function, the C scratch registers
; d0,d1,d2,a0,a1 do not have to be saved
SAVE_ALL_REGISTERS: .macro
        lea.l   -4*16(sp),sp            ; make room on the stack
        movem.l d3-d7,4*3(sp)           ; save regs (can optimize and
        movem.l a2-a6,4*10(sp)          ; not save d0,d1,d2,a0,a1)
        move.w  #$4000,d0               ; set coldfire stack frame
        swap    d0        
        move.w  sr,d0        
        move.l  d0,4*15(sp)             ; save SR and format                                         
        .endm

SAVE_DSP_REGISTERS: .macro TD_REG TEMPA_REG
        .if PSP_HAS_DSP

        move.l  TD_STACK_BASE(TD_REG),TEMPA_REG
        
        .if PSP_HAS_EMAC
        
;        move.l  ACCext01,d1
        .word   0xAB81
        move.l  d1,TD_DSP_ACCEXT01(TEMPA_REG)
;        move.l  ACCext23,d1
        .word   0xAF81
        move.l  d1,TD_DSP_ACCEXT23(TEMPA_REG)

;        move.l  ACC0,d1
        .word   0xA181
        move.l  d1,TD_DSP_ACC0(TEMPA_REG)
;        move.l  ACC1,d1
        .word   0xA381
        move.l  d1,TD_DSP_ACC1(TEMPA_REG)
;        move.l  ACC2,d1
        .word   0xA581
        move.l  d1,TD_DSP_ACC2(TEMPA_REG)
;        move.l  ACC3,d1
        .word   0xA781
        move.l  d1,TD_DSP_ACC3(TEMPA_REG)

        move.l  MACSR,d1
        move.l  d1,TD_DSP_MACSR(TEMPA_REG)
;        move.l  MASK,d1
        .word   0xAD81
        move.l  d1,TD_DSP_MASK(TEMPA_REG)
        
        .else	; PSP_HAS_EMAC
        
		move.l	ACC, d1
		move.l 	d1, TD_DSP_ACC0(TEMPA_REG)
        
        .endif	; PSP_HAS_EMAC
        
        .endif	; PSP_HAS_DSP
        
        .endm
        
; This macro restores the context for a task
RESTORE_ALL_REGISTERS: .macro
        movem.l  (sp),a0-a6/d0-d7        ; restore regs
        add.l    #60,sp
        .endm

RESTORE_DSP_REGISTERS: .macro TD_REG TEMPA_REG
        .if PSP_HAS_DSP
        
        move.l  TD_STACK_BASE(TD_REG),TEMPA_REG
        
        .if PSP_HAS_EMAC

        move.l  TD_DSP_ACC0(TEMPA_REG),d0
;        move.l  d0,ACC0
        .word   0xA100
        move.l  TD_DSP_ACC1(TEMPA_REG),d0
;        move.l  d0,ACC1
        .word   0xA300
        move.l  TD_DSP_ACC2(TEMPA_REG),d0
;        move.l  d0,ACC2
        .word   0xA500
        move.l  TD_DSP_ACC3(TEMPA_REG),d0
;        move.l  d0,ACC3
        .word   0xA700

        move.l  TD_DSP_ACCEXT01(TEMPA_REG),d0
;        move.l  d0,ACCEXT01
        .word   0xAB00
        move.l  TD_DSP_ACCEXT23(TEMPA_REG),d0
;        move.l  d0,ACCext23
        .word   0xAF00

        move.l  TD_DSP_MASK(TEMPA_REG),d0
;        move.l  d0,MASK
        .word   0xAD00
        move.l  TD_DSP_MACSR(TEMPA_REG),d0
        move.l  d0,MACSR
		
        .else	; PSP_HAS_EMAC
        
		move.l	TD_DSP_ACC0(TEMPA_REG), d0
		move.l	d0, ACC
        
        .endif	; PSP_HAS_EMAC
        
        .endif	; PSP_HAS_DSP
        .endm

SAVE_ISR_REGISTERS: .macro
        lea      -14*4(sp),sp           ; make room on the stack
        move.l   14*4(sp),(sp)          ; move D0 to proper location in
                                        ; saved stack frame
        movem.l  d1-d2,4(sp)            ; save isr registers (just scratch and d2)
        movem.l  a0-a2,4*8(sp)
        .endm

RESTORE_ISR_REGISTERS: .macro
        movem.l (sp),d0-d2
        movem.l 8*4(sp),a0-a2
        lea.l   4*15(sp),sp
        .endm

SAVE_REST_ISR_REGISTERS: .macro
        movem.l d3-d7,3*4(sp)           ; save rest
        movem.l a3-a6,11*4(sp) 
        .endm
                
RESTORE_REST_ISR_REGISTERS: .macro
        movem.l  (sp),a0-a6/d0-d7       ; restore regs
        add.l    #60,sp
        .endm

; This macro calls the kernel logging function, if logging enabled
KLOG:   .macro KDATA,KLOG_FUNCTION
        .if MQX_KERNEL_LOGGING
        btst.b  #0,KD_LOG_CONTROL+3(KDATA)
        beq.b   no_log\@
        jsr     KLOG_FUNCTION
no_log\@:
        .endif
        .endm
        
;------------------------------------------------------------------------
;                         EXTERNAL REFERENCES
;

        .if MQX_KERNEL_LOGGING
        .extern   __klog_block_internal
        .extern   __klog_execute_scheduler_internal
        .extern   __klog_yield_internal
        .extern   __klog_context_switch_internal
        .extern   __klog_isr_start_internal
        .extern   __klog_isr_end_internal
        .endif

        .extern   __mqx_kernel_data
; Start CR 1169
        .extern	  __mqx_system_stack       
; End CR 1169

; EOF
