/**HEADER********************************************************************
* 
* Copyright (c) 2008 Freescale Semiconductor;
* All Rights Reserved                       
*
*************************************************************************** 
*
* THIS SOFTWARE IS PROVIDED BY FREESCALE "AS IS" AND ANY EXPRESSED OR 
* IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES 
* OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  
* IN NO EVENT SHALL FREESCALE OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
* INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
* (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR 
* SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) 
* HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
* STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING 
* IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
* THE POSSIBILITY OF SUCH DAMAGE.
*
**************************************************************************
*
* $FileName: mcf522xx_dev_main.c$
* $Version : 3.0.6.0$
* $Date    : Mar-11-2009$
*
* Comments:
*
*   This file contains the main MCF522XX USB Device Controller interface
*   functions.
*
*END************************************************************************/

#include "mqx.h"

#include "psp.h"

#include "usb_dev.h"

#include "usb_dev_types.h"

#include "devapi.h"
#include "descriptors.h"
#include "usbprv_dev.h"
#include "mcf522xx_dev_main.h"

#include <lwevent.h>
#include <lwmsgq.h>

#pragma define_section usb_bdt ".usb_bdt" RW
__declspec(usb_bdt) static uint_8 bdt[512] = { 1 };    // DO NOT REMOVE INITIALIZATION !!! bug in CW (cw 7.1) - generate wrong binary code, data


#define BDT_BASE                ((uint_32*)(bdt))
#define BD_PTR(ep, rxtx, odd)   (((uint_32)BDT_BASE) & 0xfffffe00 | ((ep & 0x0f) << 5) | ((rxtx & 1) << 4) | ((odd & 1) << 3))

extern uint_32 _BDT_BASE[];

#define BD_CTRL_RX(ep, odd)     (*((uint_32*)BD_PTR(ep, 0, odd)))
#define BD_CTRL_TX(ep, odd)     (*((uint_32*)BD_PTR(ep, 1, odd)))

#define BD_ADDR_RX(ep, odd)     (*((uint_32*)BD_PTR(ep, 0, odd) + 1))
#define BD_ADDR_TX(ep, odd)     (*((uint_32*)BD_PTR(ep, 1, odd) + 1))

#define BYTEREV16(val)			(((val) >> 8 | (val) << 8) & 0xffff)

#define EP0_PACKET_SIZE		    8

static char ep_buf_00[EP0_PACKET_SIZE];
static char ep_buf_01[EP0_PACKET_SIZE];

asm uint_32 mcf5xxx_byterev(uint_32 val);

asm uint_32 mcf5xxx_byterev(uint_32 val) {
    link a6, #0
    move.l 8(a6), d0    
    //.short   0x02c0      /* byterev.l, D0   */
    byterev.l d0
    unlk a6
    rts    
}

static uint_8 _usb_device_mcf522xx_ep_read(USB_DEV_STATE_STRUCT_PTR state_ptr, uint_8 ep_num, uint_8_ptr buf_ptr, uint_32 buf_num_bytes, uint_32_ptr bytes_read_ptr) {
   	uint_16 max_packet_size;
   	uint_8 type;
   	
   	max_packet_size = state_ptr->EP_INFO[ep_num].max_packet_size;

    type = state_ptr->EP_INFO[ep_num].type;
    
    if (type != USB_CONTROL_ENDPOINT) {
        _int_disable ();
        
        BD_ADDR_RX(ep_num, state_ptr->EP_INFO[ep_num].rx_buf_odd) = mcf5xxx_byterev((uint_32)buf_ptr);
        BD_CTRL_RX(ep_num, state_ptr->EP_INFO[ep_num].rx_buf_odd) = mcf5xxx_byterev(MCF522X_USBOTG_BD_BC(buf_num_bytes) | MCF522X_USBOTG_BD_OWN | MCF522X_USBOTG_BD_DTS | MCF522X_USBOTG_BD_DATA01(state_ptr->EP_INFO[ep_num].rx_data0));

        state_ptr->EP_INFO[ep_num].rx_data0 ^= 1;
        
        _int_enable ();
    }
    else {
        _int_disable ();
        
        BD_CTRL_RX(ep_num, state_ptr->EP_INFO[ep_num].rx_buf_odd) = mcf5xxx_byterev(MCF522X_USBOTG_BD_BC(buf_num_bytes) | MCF522X_USBOTG_BD_OWN | MCF522X_USBOTG_BD_DTS /*| MCF522X_USBOTG_BD_DATA01(state_ptr->EP_INFO[ep_num].rx_data0)*/);
        BD_CTRL_RX(ep_num, state_ptr->EP_INFO[ep_num].rx_buf_odd ^ 1) = 0;

        //state_ptr->EP_INFO[ep_num].rx_data0 = 1;
        
        _int_enable ();
    }
   
   	//*bytes_read_ptr = i;
   
   	return USB_OK;
}

static uint_8 _usb_device_mcf522xx_ep_write(USB_DEV_STATE_STRUCT_PTR state_ptr, uint_8 ep_num, uint_8_ptr buf_ptr, uint_32 buf_num_bytes, uint_32_ptr bytes_written_ptr) {
   	uint_32 i = 0;
   	uint_8 status = USB_OK;
   	uint_16 max_packet_size;
   
   	max_packet_size = state_ptr->EP_INFO[ep_num].max_packet_size;
   
   	switch (ep_num) {
	case 0:
		{
         	if (buf_num_bytes > 32) {
            /* send in multiples of MPS (32) */
            buf_num_bytes = 32;
         }

         break;
      }
      case 1:
      {
         if (buf_num_bytes > 992)
         {
            /* send in multiples of MPS (64) */
            buf_num_bytes = 992 - 32;
         }
         
         break;
      }
      case 2:
      {
         if (buf_num_bytes > 512)
         {
            /* keyed to disk example - send in multiples of MPS (64) */
            buf_num_bytes = 512;
         }

         break;
      }
      default:
      {
      }
   }   
   
   
    if (buf_num_bytes > max_packet_size) {
        //asm(nop);
        buf_num_bytes = max_packet_size;
    }
   
    *bytes_written_ptr = buf_num_bytes;
	
	BD_ADDR_TX(ep_num, state_ptr->EP_INFO[ep_num].tx_buf_odd) = mcf5xxx_byterev((uint_32)buf_ptr);
	BD_CTRL_TX(ep_num, state_ptr->EP_INFO[ep_num].tx_buf_odd) = mcf5xxx_byterev(MCF522X_USBOTG_BD_BC(buf_num_bytes) | MCF522X_USBOTG_BD_OWN | MCF522X_USBOTG_BD_DTS | MCF522X_USBOTG_BD_DATA01(state_ptr->EP_INFO[ep_num].tx_data0));
	
	state_ptr->EP_INFO[ep_num].tx_data0 ^= 1;
	
    return USB_OK;
}

static uint_8 _usb_device_mcf522xx_setup_ep_bufs(USB_DEV_STATE_STRUCT_PTR state_ptr, uint_8_ptr  desc_array, \
	uint_32 desc_array_size, uint_16 desc_addr_ptr, uint_16_ptr desc_addr_ptr_ptr) {
	
    uint_32 desc_index = 0;
    uint_8 bLength;
    uint_8 bDescriptorType;
    uint_8 cfg_num;
    uint_8 if_num;
    uint_8 ep_num, ep;
    uint_8 ep_fifo = 0;
    uint_8 dev_bMaxPacketSize0;
    uint_8 usb_ep_sr_max;
    uint_8 dev_bNumConfigurations;
    uint_16 ep_buf_addr_ptr;
    uint_8 usb_ep_sr_dir;
    uint_8 usb_ep_sr_typ;
    uint_8 cfg_bConfigurationValue;
    uint_8 cfg_bNumInterfaces;
    uint_8 if_bInterfaceNumber;
    uint_8 if_bAlternateSetting;
    uint_8 if_bNumEndpoints;
    uint_8 ep_bmAttributes;
    uint_8 ep_bEndpointAddress;
    uint_8 ep_wMaxPacketSizeL;
    uint_8 ep_wMaxPacketSizeH;
    uint_16 ep_wMaxPacketSize;

    VMCF522X_USBOTG_STRUCT_PTR usb_ptr = _bsp_get_usb_base(0);

   
   /* determine the descriptor size and descriptor type */
   bLength = desc_array[desc_index + 0];
   bDescriptorType = desc_array[desc_index + 1];
   
   if (bDescriptorType != USB_DESC_TYPE_DEV)
   {
      return USBERR_ERROR;
   }
   
   /* determine the maximum packet size of endpoint zero */
   dev_bMaxPacketSize0 = desc_array[desc_index + 7];
   
   /* setup all EPs */
   usb_ep_sr_dir = 0x0;
   usb_ep_sr_typ = 0x0;
   
   if (dev_bMaxPacketSize0 == 8)
   {
      usb_ep_sr_max = 0x0;
   }
   else if (dev_bMaxPacketSize0 == 16)
   {
      usb_ep_sr_max = 0x1;
   }
   else if (dev_bMaxPacketSize0 == 32)
   {
      usb_ep_sr_max = 0x2;
   }
   else if (dev_bMaxPacketSize0 == 64)
   {
      usb_ep_sr_max = 0x3;
   }   


  
   /* descriptor RAM address matches Freescale example code */
   ep_buf_addr_ptr = (uint_16)(((0x0 & 0x3) << 14) | ((ep_fifo & 0xF) << 0));
   
   state_ptr->EP_INFO[ep_fifo].type = usb_ep_sr_typ;
   state_ptr->EP_INFO[ep_fifo].direction = USB_RECV;
   state_ptr->EP_INFO[ep_fifo].max_packet_size = dev_bMaxPacketSize0;

    // ep 0
    state_ptr->EP_INFO[0].rx_data0 = 0;
    state_ptr->EP_INFO[0].tx_data0 = 0;
    state_ptr->EP_INFO[0].rx_buf_odd = 0;
    state_ptr->EP_INFO[0].tx_buf_odd = 0;
    
    usb_ptr->CTL |= MCF522X_USBOTG_CTL_ODD_RST;

    // ep 0 buffers
    BD_ADDR_RX(0, 0) = mcf5xxx_byterev((uint_32)ep_buf_00);
    BD_ADDR_RX(0, 1) = mcf5xxx_byterev((uint_32)ep_buf_01);
    
    usb_ptr->CTL &= ~MCF522X_USBOTG_CTL_ODD_RST;
    
    BD_CTRL_TX(0, 0) = 0;
    BD_CTRL_TX(0, 1) = 0;
    
	usb_ptr->ENDPT0 = MCF522X_USBOTG_ENDPT_EP_HSHK | MCF522X_USBOTG_ENDPT_EP_TX_EN | MCF522X_USBOTG_ENDPT_EP_RX_EN;

   ++ep_fifo;
   
   dev_bNumConfigurations = desc_array[desc_index + 17];
   
   for (cfg_num = 0; cfg_num < dev_bNumConfigurations; cfg_num++)
   {
      /* scan to next Configuration descriptor */
      for (desc_index += bLength; desc_index < desc_array_size; desc_index += bLength)
      {
         /* determine the descriptor size and descriptor type */
         bLength = desc_array[desc_index + 0];
         bDescriptorType = desc_array[desc_index + 1];
         
         if (bDescriptorType != USB_DESC_TYPE_CFG)
         {
            --cfg_num;
            continue;
         }
         else
         {
            break;
         }
      }
      
      if (!(desc_index <= desc_array_size))
      {
         return USBERR_ERROR;
      }

      cfg_bNumInterfaces = desc_array[desc_index + 4];
      cfg_bConfigurationValue = desc_array[desc_index + 5];
      
      for (if_num = 0; if_num < cfg_bNumInterfaces; if_num++)
      {
         /* scan to next Interface descriptor */
         for (desc_index += bLength; desc_index < desc_array_size; desc_index += bLength)
         {
            /* determine the descriptor size and descriptor type */
            bLength = desc_array[desc_index + 0];
            bDescriptorType = desc_array[desc_index + 1];
            
            if (bDescriptorType != USB_DESC_TYPE_IF)
            {
               --if_num;
               continue;
            }
            else
            {
               break;
            }
         }
         
         if (!(desc_index <= desc_array_size))
         {
            return USBERR_ERROR;
         }

         if_bInterfaceNumber = desc_array[desc_index + 2];
         if_bAlternateSetting = desc_array[desc_index + 3];
         if_bNumEndpoints = desc_array[desc_index + 4];
         
         for (ep_num = 0; ep_num < if_bNumEndpoints; ep_num++)
         {
            /* scan to next Endpoint descriptor */
            for (desc_index += bLength; desc_index < desc_array_size; desc_index += bLength)
            {
               /* determine the descriptor size and descriptor type */
               bLength = desc_array[desc_index + 0];
               bDescriptorType = desc_array[desc_index + 1];
               
               if (bDescriptorType != USB_DESC_TYPE_EP)
               {
                  --ep_num;
                  continue;
               }
               else
               {
                  break;
               }
            }
            
            if (desc_index == desc_array_size)
                break;

            
            if (!(desc_index <= desc_array_size))
            {
               return USBERR_ERROR;
            }

            ep_bEndpointAddress = desc_array[desc_index + 2];
            ep_bmAttributes = desc_array[desc_index + 3];
            ep_wMaxPacketSizeL = desc_array[desc_index + 4];
            ep_wMaxPacketSizeH = desc_array[desc_index + 5];
            ep_wMaxPacketSize = (uint_16)((ep_wMaxPacketSizeH << 8) |
                                 (ep_wMaxPacketSizeL));

            /* setup all EPs */
            usb_ep_sr_dir = (uint_8)((ep_bEndpointAddress & USB_DESC_EP_ENDPOINT_ADDRESS_DIR_POS) >> USB_DESC_EP_ENDPOINT_ADDRESS_DIR_SHIFT);
            usb_ep_sr_typ = (uint_8)((ep_bmAttributes & USB_DESC_EP_ATTRIBUTES_TRANSFER_TYPE_POS) >> USB_DESC_EP_ATTRIBUTES_TRANSFER_TYPE_SHIFT);

            ep = (uint_8)((ep_bEndpointAddress & USB_DESC_EP_ENDPOINT_ADDRESS_EP_NUMBER_POS) >> USB_DESC_EP_ENDPOINT_ADDRESS_EP_NUMBER_SHIFT);
            
            if (usb_ep_sr_typ != 0x1)
            {
               if (ep_wMaxPacketSize == 8)
               {
                  usb_ep_sr_max = 0x0;
               }
               else if (ep_wMaxPacketSize == 16)
               {
                  usb_ep_sr_max = 0x1;
               }
               else if (ep_wMaxPacketSize == 32)
               {
                  usb_ep_sr_max = 0x2;
               }
               else if (ep_wMaxPacketSize == 64)
               {
                  usb_ep_sr_max = 0x3;
               }
            }

            ep_buf_addr_ptr = (uint_16)(((0x3 & 0x3) << 14) | ((ep_fifo & 0xF) << 0));
            
            state_ptr->EP_INFO[ep].type = usb_ep_sr_typ;

            state_ptr->EP_INFO[ep].max_packet_size = ep_wMaxPacketSize;

            state_ptr->EP_INFO[ep].rx_data0 = 0;
            state_ptr->EP_INFO[ep].tx_data0 = 0;
            state_ptr->EP_INFO[ep].rx_buf_odd = 0;
            state_ptr->EP_INFO[ep].tx_buf_odd = 0;

            BD_CTRL_RX(ep, 0) = 0;
            BD_CTRL_RX(ep, 1) = 0;
            
            if (usb_ep_sr_dir == 0x0) {
                state_ptr->EP_INFO[ep].direction = USB_RECV;

                BD_CTRL_RX(ep, 0) = 0;
                BD_CTRL_RX(ep, 1) = 0;

                *((uint_8_ptr)((uint_32_ptr)(&usb_ptr->ENDPT0) + ep)) = MCF522X_USBOTG_ENDPT_EP_HSHK | MCF522X_USBOTG_ENDPT_EP_RX_EN | MCF522X_USBOTG_ENDPT_EP_CTL_DIS;
            }
            else {
                state_ptr->EP_INFO[ep].direction = USB_SEND;

                BD_CTRL_TX(ep, 0) = 0;//mcf5xxx_byterev(MCF522X_USBOTG_BD_OWN /*| MCF522X_USBOTG_BD_DTS*/);
                BD_CTRL_TX(ep, 1) = 0;

                *((uint_8_ptr)((uint_32_ptr)(&usb_ptr->ENDPT0) + ep)) = MCF522X_USBOTG_ENDPT_EP_HSHK | MCF522X_USBOTG_ENDPT_EP_TX_EN | MCF522X_USBOTG_ENDPT_EP_CTL_DIS;
            }

            ++ep_fifo;
         }
      }
   }

    *desc_addr_ptr_ptr = desc_addr_ptr;
    return USB_OK;
}

static void _usb_device_mcf522xx_reset_ep_state(USB_DEV_STATE_STRUCT_PTR state_ptr) {
    VMCF522X_USBOTG_STRUCT_PTR usb_ptr = _bsp_get_usb_base(0);
    _mqx_int ep;

    usb_ptr->CTL |= MCF522X_USBOTG_CTL_ODD_RST;

    usb_ptr->ADDR = 0;
    
    BD_ADDR_RX(0, 0) = mcf5xxx_byterev((uint_32)ep_buf_00);
    BD_ADDR_RX(0, 1) = mcf5xxx_byterev((uint_32)ep_buf_01);
    
    for (ep = 0; ep < USB_MAX_ENDPOINTS; ep++) {
        BD_CTRL_RX(ep, 0) = 0;
        BD_CTRL_RX(ep, 1) = 0;
    
        BD_CTRL_TX(ep, 0) = 0;
        BD_CTRL_TX(ep, 1) = 0;

        state_ptr->EP_INFO[ep].rx_data0 = 0;
        state_ptr->EP_INFO[ep].tx_data0 = 0;
        state_ptr->EP_INFO[ep].rx_buf_odd = 0;
        state_ptr->EP_INFO[ep].tx_buf_odd = 0;
        
        *((uint_8_ptr)((uint_32_ptr)(&usb_ptr->ENDPT0) + ep)) = 0;
    }
    
    usb_ptr->CTL &= ~MCF522X_USBOTG_CTL_ODD_RST;    
    
	usb_ptr->ENDPT0 = 0;

    state_ptr->DEVICE_ADDRESS = 0;	
	state_ptr->USB_STATE = USB_STATE_DEFAULT;
	
    //usb_ptr->INT_ENB &= MCF522X_USBOTG_INT_ENB_RESUME_EN;
    //usb_ptr->INT_ENB |= MCF522X_USBOTG_INT_ENB_SLEEP_EN;
}


/* USB 2.0 Setup Packet */
typedef struct setup_struct {
   uint_8      REQUESTTYPE;
   uint_8      REQUEST;
   uint_16     VALUE;
   uint_16     INDEX;
   uint_16     LENGTH;
} SETUP_STRUCT, _PTR_ SETUP_STRUCT_PTR;

static _mqx_int _set_addr(_usb_device_handle handle, SETUP_STRUCT_PTR setup_ptr) {
    _usb_device_send_data(handle, 0, NULL, 0);

    _usb_device_set_status(handle, USB_STATUS_ADDRESS, BYTEREV16(setup_ptr->VALUE));
    
    return 0;
}

static _mqx_int _get_descriptor(_usb_device_handle handle, SETUP_STRUCT_PTR setup_ptr) {
    uint_8  desc_type;
    uint_8  desc_index;
    _mqx_int res = 0;
    
    desc_type = setup_ptr->VALUE & 0xff;
    desc_index = setup_ptr->VALUE >> 8 & 0xff;
    
    switch (desc_type) {
    case USB_DESC_TYPE_DEV:
        // DEVICE descriptor
        _usb_device_send_data(handle, 0, (uchar_ptr)((pointer)&Descriptor), USB_DESC_LEN_DEV);
        break;
    case USB_DESC_TYPE_CFG:
        // CONFIGURATION descriptor
	    _usb_device_send_data(handle, 0, (uchar_ptr)((pointer)&Descriptor) + 18, (BYTEREV16(setup_ptr->LENGTH) < DescriptorSize - 18) ? BYTEREV16(setup_ptr->LENGTH) : DescriptorSize - 18);
        break;
    case USB_DESC_TYPE_DEV_QUALIFIER:
        // DEVICE QUALIFIER
		_usb_device_send_data(handle, 0, (uchar_ptr)((pointer)&Descriptor) + USB_DESC_LEN_DEV + USB_DESC_LEN_CFG + USB_DESC_LEN_IF + 9 + USB_DESC_LEN_EP, USB_DESC_LEN_DEV_QUALIFIER);
		break;
    default:
        res = -1;
    }
    
    return res;
}

static _mqx_int _setup_eval(_usb_device_handle handle, SETUP_STRUCT_PTR setup_ptr) {
    _mqx_int res = -1;

    switch (setup_ptr->REQUESTTYPE & 0x60) {
    case 0:
        switch (setup_ptr->REQUEST) {
        case 0x05:
            res = _set_addr(handle, setup_ptr);
            break;
        case 0x06:
            res = _get_descriptor(handle, setup_ptr);
            break;
        }
        break;
    }
    
    if (!res) {
        _usb_device_recv_data(handle, 0, NULL, 8);
    }
    
    return res;
}

/*FUNCTION*-------------------------------------------------------------
*
*  Function Name  : _usb_device_mcf522xx_isr
*  Returned Value : None
*  Comments       :
*        Service all the interrupts in the kirin usb hardware
*END*-----------------------------------------------------------------*/
static void _usb_device_mcf522xx_isr(USB_DEV_STATE_STRUCT_PTR state_ptr) {
    uchar status;
    VMCF522X_USBOTG_STRUCT_PTR usb_ptr = _bsp_get_usb_base(0);
    uint_32 type, tmp;
    XD_STRUCT_PTR xd_ptr;
    uchar ep_num, dir, buf_odd;
    _mqx_int i, len, stat;
    uint_16 desc_addr_ptr;
    
    while (1) {
        status = (uint_8)(usb_ptr->INT_STAT & usb_ptr->INT_ENB);
        
        if (!status) {
            //_int_enable();
            break;
        }
        
        //_int_disable();
        
        if (status & MCF522X_USBOTG_INT_STAT_TOK_DNE) {
            stat = usb_ptr->STAT;
            usb_ptr->INT_STAT = MCF522X_USBOTG_INT_STAT_TOK_DNE;

            ep_num = stat >> 4 & 0x0f;
            dir = (stat & MCF522X_USBOTG_STAT_TX) ? 1 : 0;
            buf_odd = (stat & MCF522X_USBOTG_STAT_ODD) ? 1 : 0;

            //printf("ep:%d dir:%d odd:%d stat(0x%.2x)\n", ep_num, dir, buf_odd, stat);

/*            tmp = mcf5xxx_byterev(BD_CTRL_RX(ep_num, 0));
            printf("rx0: 0x%.8x -> own:0x%.2x len:%d p:0x%x\n", tmp, tmp & MCF522X_USBOTG_BD_OWN, tmp >> 16 & 0x3ff, tmp >> 2 & 0x0f);
            tmp = mcf5xxx_byterev(BD_CTRL_RX(ep_num, 1));
            printf("rx1: 0x%.8x -> own:0x%.2x len:%d p:0x%x\n", tmp, tmp & MCF522X_USBOTG_BD_OWN, tmp >> 16 & 0x3ff, tmp >> 2 & 0x0f);

            tmp = mcf5xxx_byterev(BD_CTRL_TX(ep_num, 0));
            printf("tx0: 0x%.8x -> own:0x%.2x len:%d p:0x%x\n", tmp, tmp & MCF522X_USBOTG_BD_OWN, tmp >> 16 & 0x3ff, tmp >> 2 & 0x0f);
            tmp = mcf5xxx_byterev(BD_CTRL_TX(ep_num, 1));
            printf("tx1: 0x%.8x -> own:0x%.2x len:%d p:0x%x\n", tmp, tmp & MCF522X_USBOTG_BD_OWN, tmp >> 16 & 0x3ff, tmp >> 2 & 0x0f);
*/
            USB_XD_QUEUE_GET_HEAD(&state_ptr->EP_INFO[ep_num].xd_queue, &xd_ptr);
            
            if (xd_ptr == NULL) {
                asm(nop);
                //printf("XD: 0x%p\n", xd_ptr);
            }
            else {
                //printf("XD: 0x%p\n", xd_ptr);
                //printf("XD: e:%d d:%d t:%d l:%d,%d\n", xd_ptr->EP_NUM, xd_ptr->BDIRECTION, xd_ptr->EP_TYPE, xd_ptr->WTOTALLENGTH, xd_ptr->WSOFAR);
                
                
                if (xd_ptr->EP_NUM != ep_num || xd_ptr->BDIRECTION != dir) {
                /*
                    tmp = mcf5xxx_byterev(BD_CTRL_RX(ep_num, 0));
                    printf("RX: 0x%x", tmp);
                    tmp = mcf5xxx_byterev(BD_CTRL_RX(ep_num, 1));
                    printf(" 0x%x\n", tmp);

                    tmp = mcf5xxx_byterev(BD_CTRL_TX(ep_num, 0));
                    printf("TX: 0x%x", tmp);
                    tmp = mcf5xxx_byterev(BD_CTRL_TX(ep_num, 1));
                    printf(" 0x%x\n", tmp);*/
/*                        USB_XD_QUEUE_DEQUEUE (&state_ptr->EP_INFO[ep_num].xd_queue, &xd_ptr);
                        _usb_device_free_XD (state_ptr, xd_ptr);
            USB_XD_QUEUE_GET_HEAD(&state_ptr->EP_INFO[ep_num].xd_queue, &xd_ptr);
  */
                    asm(nop);
                }
            }
            
            if (dir) {
                // tx
                state_ptr->EP_INFO[ep_num].tx_buf_odd = buf_odd ^ 1;                
                
                if (xd_ptr == NULL) {
                    _usb_device_call_service(state_ptr, ep_num, FALSE, stat >> 3 & 1, 0, 0);
                }
                else {
                
                    if (xd_ptr->WSOFAR != xd_ptr->WTOTALLENGTH) {
                        _usb_dci_mcf522xx_send(state_ptr, xd_ptr);
                    }
                    else {
                        
                        xd_ptr->BSTATUS = USB_STATUS_IDLE;
                        
                        if (state_ptr->USB_STATE == USB_STATE_DEFAULT) {
                            _usb_dci_mcf522xx_set_addr(state_ptr, state_ptr->DEVICE_ADDRESS);
                        }
                            
                        _usb_device_call_service(state_ptr, ep_num, FALSE, stat >> 3 & 1, 0, 0);

                        USB_XD_QUEUE_DEQUEUE (&state_ptr->EP_INFO[ep_num].xd_queue, &xd_ptr);
                        _usb_device_free_XD (state_ptr, xd_ptr);
                    }
                
                }
            }
            else {
                // rx
                len = mcf5xxx_byterev(BD_CTRL_RX(ep_num, buf_odd)) >> 16 & 0x3ff;
                
                BD_CTRL_RX(ep_num, buf_odd) &= ~mcf5xxx_byterev(MCF522X_USBOTG_BD_OWN | MCF522X_USBOTG_BD_STALL);
                usb_ptr->CTL &= ~MCF522X_USBOTG_CTL_TXSUSPEND_TOKENBUSY;

                state_ptr->EP_INFO[ep_num].rx_buf_odd ^= 1;

                if (ep_num == 0 && len) {
                    state_ptr->EP_INFO[ep_num].tx_data0 = 1;
                    state_ptr->EP_INFO[ep_num].rx_data0 = 1;

                    xd_ptr->BSTATUS = USB_STATUS_IDLE;
                
                    if (_setup_eval(state_ptr, (buf_odd == 0) ? ep_buf_00 : ep_buf_01)) {
                        _usb_device_call_service(state_ptr, ep_num, (ep_num == 0), stat >> 3 & 1, 0, 0);
                    }
                    
                    USB_XD_QUEUE_DEQUEUE (&state_ptr->EP_INFO[ep_num].xd_queue, &xd_ptr);
                    _usb_device_free_XD (state_ptr, xd_ptr);
                }
                else {
                    xd_ptr->WSOFAR += mcf5xxx_byterev(BD_CTRL_RX(ep_num, buf_odd)) >> 16 & 0x3ff;

                    if (xd_ptr->WSOFAR < xd_ptr->WTOTALLENGTH && len) {
                        _usb_dci_mcf522xx_recv(state_ptr, xd_ptr);
                    }
                    else {
                        xd_ptr->BSTATUS = USB_STATUS_IDLE;
                        
                        _usb_device_call_service(state_ptr, ep_num, FALSE, stat >> 3 & 1, xd_ptr->WSTARTADDRESS, xd_ptr->WTOTALLENGTH);

                        USB_XD_QUEUE_DEQUEUE(&state_ptr->EP_INFO[ep_num].xd_queue, &xd_ptr);
                        _usb_device_free_XD(state_ptr, xd_ptr);
                    }
                }
                //}
            }
        }
        else {                
        if (status & MCF522X_USBOTG_INT_STAT_USB_RST) {   // usb reset
            /* free all queued XDs on endpoints */         
            for (i = 0; i < USB_MAX_ENDPOINTS; i++) {
                while (1) {
                    USB_XD_QUEUE_DEQUEUE (&state_ptr->EP_INFO[i].xd_queue, &xd_ptr);
                
                    if (xd_ptr == NULL)
                        break;
               
                    _usb_device_free_XD ((_usb_device_handle)state_ptr, xd_ptr);
                }
            }

            _usb_device_mcf522xx_reset_ep_state(state_ptr);
            _usb_device_mcf522xx_setup_ep_bufs(state_ptr, Descriptor, DescriptorSize, desc_addr_ptr, &desc_addr_ptr);

            if (_usb_device_call_service(state_ptr, USB_SERVICE_BUS_RESET, FALSE, FALSE, 0, 0) != USB_OK) {
            }

            //stat = usb_ptr->STAT;            
            usb_ptr->INT_STAT = 0xff;
            status = 0;
        }

        if (status & MCF522X_USBOTG_INT_STAT_SLEEP) {
            usb_ptr->INT_STAT = MCF522X_USBOTG_INT_STAT_SLEEP;        
        
            //printf("sleep\n");
        
//            usb_ptr->INT_ENB &= ~MCF522X_USBOTG_INT_ENB_SLEEP_EN;
//            usb_ptr->INT_ENB |= MCF522X_USBOTG_INT_ENB_RESUME_EN;
        }
        
        if (status & MCF522X_USBOTG_INT_STAT_RESUME) {
            usb_ptr->INT_STAT = MCF522X_USBOTG_INT_STAT_RESUME;        
            
            //printf("resume\n");

        
//            usb_ptr->INT_ENB &= MCF522X_USBOTG_INT_ENB_RESUME_EN;
//            usb_ptr->INT_ENB |= MCF522X_USBOTG_INT_ENB_SLEEP_EN;
        }
        
        if (status & MCF522X_USBOTG_INT_STAT_ERROR) {
            usb_ptr->INT_STAT = MCF522X_USBOTG_INT_STAT_ERROR;        
        }
                
        if (status & MCF522X_USBOTG_INT_STAT_STALL) {
            usb_ptr->INT_STAT = MCF522X_USBOTG_INT_STAT_STALL;
            stat = usb_ptr->STAT;
            ep_num = stat >> 4 & 0x0f;

            BD_CTRL_RX(ep_num, 0) = 0;
            BD_CTRL_RX(ep_num, 1) = 0;
            BD_CTRL_TX(ep_num, 0) = 0;
            BD_CTRL_TX(ep_num, 1) = 0;
            
            _usb_device_recv_data(state_ptr, 0, 0, 8);

        }        
        }

        //usb_ptr->INT_STAT = (status);// & ~MCF522X_USBOTG_INT_STAT_TOK_DNE);

    };
}

/*FUNCTION*-------------------------------------------------------------
*
*  Function Name  : _usb_dci_mcf522xx_send
*  Returned Value : USB_OK or error code
*  Comments       : Sends data. Non-blocking.
*
*END*-----------------------------------------------------------------*/
uint_8 _usb_dci_mcf522xx_send(_usb_device_handle handle, XD_STRUCT_PTR xd_ptr) {
   	uint_8 status;
   	USB_DEV_STATE_STRUCT_PTR state_ptr = (USB_DEV_STATE_STRUCT_PTR)handle;
   	uint_8 type, ep_num;
   	uint_32 buf_num_bytes;
   	XD_STRUCT_PTR temp_xd_ptr;
   	//uint_8 i;
   	
    ep_num = xd_ptr->EP_NUM;
    buf_num_bytes = (uint_32)(xd_ptr->WTOTALLENGTH - xd_ptr->WSOFAR);

    type = state_ptr->EP_INFO[ep_num].type;
    
    state_ptr->PENDING_XD_PTR = NULL;
    
    if (xd_ptr->WSOFAR == 0) {
   	    USB_XD_QUEUE_ENQUEUE(&state_ptr->EP_INFO[xd_ptr->EP_NUM].xd_queue, xd_ptr);
    }

    if (buf_num_bytes == 0) {
   	    status = _usb_device_mcf522xx_ep_write(state_ptr, xd_ptr->EP_NUM, xd_ptr->WSTARTADDRESS, buf_num_bytes, &buf_num_bytes);
      
      	if (status != USB_OK) {
         	USB_XD_QUEUE_DEQUEUE (&state_ptr->EP_INFO[xd_ptr->EP_NUM].xd_queue, &temp_xd_ptr);
         	return (uint_8)status;
      	}
      
   	    state_ptr->PENDING_XD_PTR = xd_ptr;
   	    
      	return USB_OK;
   	}

   	status = _usb_device_mcf522xx_ep_write(state_ptr, xd_ptr->EP_NUM, (uint_8_ptr)xd_ptr->WSTARTADDRESS + xd_ptr->WSOFAR, buf_num_bytes, &buf_num_bytes);

   	if (status != USB_OK) {
      	USB_XD_QUEUE_DEQUEUE (&state_ptr->EP_INFO[xd_ptr->EP_NUM].xd_queue, &temp_xd_ptr);
   	}
   
   	xd_ptr->WSOFAR += buf_num_bytes;
   	
   	state_ptr->PENDING_XD_PTR = xd_ptr;

    return USB_OK;
}

/*FUNCTION*-------------------------------------------------------------
*
*  Function Name  : _usb_dci_mcf522xx_recv
*  Returned Value : USB_OK or error code
*  Comments       : Receives data. Non-blocking.
*
*END*-----------------------------------------------------------------*/
uint_8 _usb_dci_mcf522xx_recv(_usb_device_handle handle, XD_STRUCT_PTR xd_ptr) {
    USB_DEV_STATE_STRUCT_PTR state_ptr = (USB_DEV_STATE_STRUCT_PTR)handle;
    uint_8 ep_num;
    uint_8 type;
    uint_32 status;
   	uint_32 buf_num_bytes;    
   	XD_STRUCT_PTR temp_xd_ptr;
       
    buf_num_bytes = (uint_32)(xd_ptr->WTOTALLENGTH - xd_ptr->WSOFAR);
    ep_num = xd_ptr->EP_NUM;
    type = state_ptr->EP_INFO[ep_num].type;

    state_ptr->PENDING_XD_PTR = NULL;
    
    if (xd_ptr->WSOFAR == 0) {
        USB_XD_QUEUE_ENQUEUE(&state_ptr->EP_INFO[xd_ptr->EP_NUM].xd_queue, xd_ptr);
    }

    if (buf_num_bytes > state_ptr->EP_INFO[ep_num].max_packet_size) {
        buf_num_bytes = state_ptr->EP_INFO[ep_num].max_packet_size;
    }
   
    status = _usb_device_mcf522xx_ep_read(handle, ep_num, (uint_8_ptr)((uint_32)xd_ptr->WSTARTADDRESS + (uint_32)xd_ptr->WSOFAR), buf_num_bytes, (uint_32_ptr)&buf_num_bytes);

   	if (status != USB_OK) {
      	USB_XD_QUEUE_DEQUEUE(&state_ptr->EP_INFO[xd_ptr->EP_NUM].xd_queue, &temp_xd_ptr);
   	}
   	else {
        state_ptr->PENDING_XD_PTR = xd_ptr;
    }
   
    return USB_OK;
}

/*FUNCTION*-------------------------------------------------------------
*
*  Function Name  : _usb_dci_mcf522xx_cancel
*  Returned Value : USB_OK or error code
*  Comments       :
*        Cancels all pending transfers on an endpoint.
*
*END*-----------------------------------------------------------------*/
uint_8 _usb_dci_mcf522xx_cancel(_usb_device_handle handle, uint_8 ep_num, uint_8 direction) {
    return USB_OK;
}

/*FUNCTION*-------------------------------------------------------------
*
*  Function Name  : _usb_dci_mcf522xx_set_addr
*  Returned Value : USB_OK or error code
*  Comments       :
*        Set device address.
*
*END*-----------------------------------------------------------------*/
uint_8 _usb_dci_mcf522xx_set_addr(_usb_device_handle handle, uint_8 addr) {
    VMCF522X_USBOTG_STRUCT_PTR usb_ptr = _bsp_get_usb_base(0);

    if (addr) {    
        usb_ptr->ADDR = addr;
        ((USB_DEV_STATE_STRUCT_PTR)handle)->USB_STATE = USB_STATE_ADDRESS;
    }
    
    return USB_OK;
}

/*FUNCTION*-------------------------------------------------------------
*
*  Function Name  : _usb_dci_mcf522xx_shutdown
*  Returned Value : USB_OK or error code
*  Comments       :
*        Shuts down the MCF522XX Device Controller
*
*END*-----------------------------------------------------------------*/
uint_8 _usb_dci_mcf522xx_shutdown(_usb_device_handle handle) {
    return USB_OK;
}

/*FUNCTION*-------------------------------------------------------------
*
*  Function Name  : _usb_dci_mcf522xx_get_setup_data
*  Returned Value : USB_OK or error code
*  Comments       : Reads the Setup data from the 8-byte setup buffer
*
*END*-----------------------------------------------------------------*/
uint_8 _usb_dci_mcf522xx_get_setup_data(_usb_device_handle handle, uint_8 ep_num, uchar_ptr buffer_ptr) {
   	USB_DEV_STATE_STRUCT_PTR state_ptr = (USB_DEV_STATE_STRUCT_PTR)handle;
	
   	if (ep_num == 0) {
        if (state_ptr->EP_INFO[0].rx_buf_odd)
            _mem_copy(ep_buf_00, buffer_ptr, 8);
        else
            _mem_copy(ep_buf_01, buffer_ptr, 8);
    }
    else
        return USBERR_ERROR;
    
	*((uint_16*)&buffer_ptr[2]) = BYTEREV16(*((uint_16*)&buffer_ptr[2]));
	*((uint_16*)&buffer_ptr[4]) = BYTEREV16(*((uint_16*)&buffer_ptr[4]));
	*((uint_16*)&buffer_ptr[6]) = BYTEREV16(*((uint_16*)&buffer_ptr[6]));
	
   	return USB_OK;
}

/*FUNCTION*-------------------------------------------------------------
*
*  Function Name  : _usb_dci_mcf522xx_assert_resume
*  Returned Value : USB_OK or error code
*  Comments       :
*        Resume signalling for remote wakeup
*
*END*-----------------------------------------------------------------*/
uint_8 _usb_dci_mcf522xx_assert_resume(_usb_device_handle handle) {
    return USB_OK;
}

/*FUNCTION*-------------------------------------------------------------
*
*  Function Name  : _usb_dci_mcf522xx_stall_endpoint
*  Returned Value : USB_OK or error code
*  Comments       :
*        Stalls the specified endpoint
*
*END*-----------------------------------------------------------------*/
uint_8 _usb_dci_mcf522xx_stall_endpoint(_usb_device_handle handle, uint_8 ep_num, uint_8 direction) {
    USB_DEV_STATE_STRUCT_PTR state_ptr = (USB_DEV_STATE_STRUCT_PTR)handle;
    
    if (direction) {
        BD_CTRL_RX(ep_num, state_ptr->EP_INFO[ep_num].rx_buf_odd) = mcf5xxx_byterev(MCF522X_USBOTG_BD_OWN | MCF522X_USBOTG_BD_STALL);
    }
    else {
        BD_CTRL_TX(ep_num, state_ptr->EP_INFO[ep_num].tx_buf_odd) = mcf5xxx_byterev(MCF522X_USBOTG_BD_OWN | MCF522X_USBOTG_BD_STALL);
    }
    
    return USB_OK;
}

/*FUNCTION*-------------------------------------------------------------
*
*  Function Name  : _usb_dci_mcf522xx_init
*  Returned Value : USB_OK or error code
*  Comments       : Initializes the USB device controller.
*
*END*-----------------------------------------------------------------*/
uint_8 _usb_dci_mcf522xx_init(uint_8 devnum, _usb_device_handle handle) {
	USB_STATUS status = USB_OK;
	USB_DEV_STATE_STRUCT_PTR state_ptr = handle;
	VMCF522X_USBOTG_STRUCT_PTR usb_ptr = _bsp_get_usb_base(0);
	
	uint_16 desc_addr_ptr;

	// install isr
	if (!(USB_install_isr(_bsp_get_usb_vector(0), _usb_device_mcf522xx_isr, state_ptr)))
		return USBERR_INSTALL_ISR;
	
    usb_ptr->USB_CTRL = /*MCF522X_USBOTG_USB_CTRL_PDE |*/ MCF522X_USBOTG_USB_CTRL_CLK_SRC_OSCCLK;
    
    //usb_ptr->CTL |= MCF522X_USBOTG_CTL_ODD_RST;

	// setup endpoints    
    usb_ptr->ADDR = 0;
    
	USB_memzero(BDT_BASE, 512);

    // setup buffers
    status = _usb_device_mcf522xx_setup_ep_bufs(state_ptr, Descriptor, DescriptorSize, desc_addr_ptr, &desc_addr_ptr);

   	if (status != USB_OK) {
      	return (uint_8)status;
   	}
   	
   	_usb_device_mcf522xx_reset_ep_state(state_ptr);
	
	
	usb_ptr->BDT_PAGE_01 = (uint_8)((uint_32)BDT_BASE >> 8);
    usb_ptr->BDT_PAGE_02 = (uint_8)((uint_32)BDT_BASE >> 16);
    usb_ptr->BDT_PAGE_03 = (uint_8)((uint_32)BDT_BASE >> 24);
    
	usb_ptr->INT_STAT = 0xff;
    usb_ptr->INT_ENB = MCF522X_USBOTG_INT_ENB_SLEEP_EN | MCF522X_USBOTG_INT_ENB_TOK_DNE_EN | MCF522X_USBOTG_INT_ENB_ERROR_EN |
        MCF522X_USBOTG_INT_ENB_USB_RST_EN | MCF522X_USBOTG_INT_ENB_STALL_EN | MCF522X_USBOTG_INT_ENB_RESUME_EN;

    usb_ptr->CTL = MCF522X_USBOTG_CTL_USB_EN_SOF_EN;

    usb_ptr->OTG_CTRL = MCF522X_USBOTG_OTG_CTRL_DP_HIGH | MCF522X_USBOTG_OTG_CTRL_OTG_EN;
    
	return status;    
}
